<?php

/**
 * TransApiHandler class
 *
 * Handles API requests for invoice data
 */
class TransApiHandler extends CI_Controller
{
    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Handle invalid request method
     */
    private function handleInvalidRequestMethod()
    {
        $this->output->set_status_header(405);
        $this->output->set_content_type('application/json');
        $this->output->set_output(json_encode(['error' => 'Invalid request method']));
    }

    /**
     * Handle invalid JSON data
     */
    private function handleInvalidJsonData()
    {
        $this->output->set_status_header(400);
        $this->output->set_content_type('application/json');
        $this->output->set_output(json_encode(['error' => 'Invalid JSON data']));
    }

    /**
     * Handle failed request
     */
    private function handleFailedRequest()
    {
        $this->output->set_status_header(500);
        $this->output->set_content_type('application/json');
        $this->output->set_output(json_encode(['error' => 'Failed to send request']));
    }

    /**
     * Handle failed insertion
     */
    private function handleFailedInsertion()
    {
        $this->output->set_status_header(500);
        $this->output->set_content_type('application/json');
        $this->output->set_output(json_encode(['error' => 'Failed to insert data']));
    }

    /**
     * Handle successful response
     */
    private function handleSuccessResponse()
    {
        $this->output->set_status_header(200);
        $this->output->set_content_type('application/json');
        $this->output->set_output(json_encode(['message' => 'Data inserted successfully']));
    }

    /**
     * API endpoint for invoice data
     */
    public function api_invoice_data()
    {
        // Check request method
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            $this->handleInvalidRequestMethod();
            return;
        }

        // Get JSON data from request
        $json_data = json_decode($this->input->raw_input_stream, true);
       // $json_data = json_decode($this->input->raw_input_stream, JSON_OBJECT_AS_ARRAY);

        // Check if JSON data is empty
        if (empty($json_data)) {
            $this->handleInvalidJsonData();
            return;
        }

        // Check for JSON errors
        if (json_last_error() !== JSON_ERROR_NONE) {
            $this->handleInvalidJsonData();
            return;
        }

      

        // Prepare API data
        $api_invoice_data = [
            'inv_branch_id' => $json_data['inv_branch_id'],
            'inv_id' => $json_data['inv_id'],
            'inv_name' => $json_data['inv_name'],
            'inv_invoice_number' => $json_data['inv_invoice_number'],
            'inv_invoice_type' => $json_data['inv_invoice_type'],
            'inv_ra_no' => $json_data['inv_ra_no'],
            'inv_lpo_number' => $json_data['inv_lpo_number'],
            'inv_customer_id' => $json_data['inv_customer_id'],
            'inv_customer_name' => $json_data['inv_customer_name'],
            'inv_customer_type' => $json_data['inv_customer_type'],
            'inv_payment_mode' => $json_data['inv_payment_mode'],
            'inv_car_no' => $json_data['inv_car_no'],
            'inv_taxable_amt' => $json_data['inv_taxable_amt'],
            'inv_invoice_tax' => $json_data['inv_invoice_tax'],
            'inv_invoice_net_amt' => $json_data['inv_invoice_net_amt'],
            'inv_invoice_advance' => $json_data['inv_invoice_advance'],
            'inv_description' => $json_data['inv_description'],
            'inv_invoice_amt' => $json_data['inv_invoice_amt'],
            'inv_invoice_item_tax' => $json_data['inv_invoice_item_tax'],
            'inv_invoice_item_net_amt' => $json_data['inv_invoice_item_net_amt'],
        ];

          // Set API URL
          $server_url = 'http://150.150.1.100:8080/nabas/transAPI/TransApiHandler/api_invoice_data'; // Replace with the correct server URL
        // Send POST request to API
        $response = $this->sendPostRequest($server_url, $api_invoice_data);

        // Check if request failed
        if ($response === false) {
            $this->handleFailedRequest();
            return;
        }

        // Decode response data
        $responseData = json_decode($response, true);
        print_r($responseData);
        // Check if response data is empty
        if (empty($responseData)) {
            $this->handleInvalidJsonData();
            return;
        }

        // Insert data into database
        if ($this->insertInvoiceData($api_invoice_data)) {
            $this->handleSuccessResponse();
        } else {
            $this->handleFailedInsertion();
        }
    }

    /**
     * Insert invoice data into database
     *
     * @param array $data
     * @return bool
     */
    private function insertInvoiceData($data)
    {
        $this->db->insert('invoice_api', $data);
        return $this->db->affected_rows() > 0;
    }

    /**
     * Send POST request to API
     *
     * @param string $url
     * @param array $data
     * @return string|false
     */

     private function sendPostRequest($url, $data)
{
    try {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));

        print_r($data); // Print out the input data
        print_r(curl_getinfo($ch, CURLINFO_HEADER_OUT)); // Print out the API request headers

        $response = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $error = curl_error($ch);

        curl_close($ch);

        if ($http_code !== 200) {
            log_message('error', "Failed to send request to $url. HTTP status code: $http_code. Error: $error");
            return false;
        }

        if ($error) {
            log_message('error', "An error occurred while sending the request: $error");
            return false;
        }

        return $response;
    } catch (Exception $e) {
        log_message('error', "An error occurred while sending the request: " . $e->getMessage());
        return false;
    }
}

//      private function sendPostRequest($url, $data)
// {
//     try {
//         $ch = curl_init($url);
//         curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
//         curl_setopt($ch, CURLOPT_POST, true);
//         curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
//         curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));

//         $response = curl_exec($ch);
//         $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
//         $error = curl_error($ch);

//         curl_close($ch);

//         if ($http_code !== 200) {
//             log_message('error', "Failed to send request to $url. HTTP status code: $http_code. Error: $error");
//             return false;
//         }

//         if ($error) {
//             log_message('error', "An error occurred while sending the request: $error");
//             return false;
//         }

//         return $response;
//     } catch (Exception $e) {
//         log_message('error', "An error occurred while sending the request: " . $e->getMessage());
//         return false;
//     }
// }



    // private function sendPostRequest($url, $data)
    // {
    //     $ch = curl_init($url);
    //     curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    //     curl_setopt($ch, CURLOPT_POST, true);
    //     curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
    //     curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));

         
    //     $response = curl_exec($ch);
    //     $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    //     $error = curl_error($ch);

    //     curl_close($ch);

    //     if ($http_code !== 200) {
    //         log_message('error', "Failed to send request to $url. HTTP status code: $http_code. Error: $error");
    //         return false;
    //     }

    //     return $response;
    // }
}